@@ -22,7 +22,6 @@ import pyqtgraph as pg
 from pyqtgraph.Qt import QtWidgets, QtCore, QtGui, QT_LIB
 from pyqtgraph.parametertree import Parameter, ParameterTree
 from tqdm import tqdm
-import cv2 as cv
 
 from .params import PrjParam
 from .typeoverloads import FilePath
@@ -33,9 +32,11 @@ from .constants import PrjEnums
 # -----
 # import yaml
 # from yaml import YAMLError
-# def yamlLoad(stream, loader=yaml.UnsafeLoader):
+# loader = yaml.UnsafeLoader
+# dumper = yaml.Dumper
+# def yamlLoad(stream):
 #   return yaml.load(stream, loader)
-# def yamlDump(data, stream, dumper=yaml.Dumper):
+# def yamlDump(data, stream):
 #   return yaml.dump(data, stream, dumper, sort_keys=False)
 
 # -----
@@ -680,7 +681,7 @@ def makeCli(func: t.Callable, convertArgs=True, parserKwargs: dict=None, **procK
     arg = parser.add_argument(f'--{ch["name"]}', required=ch['value'] is RunOpts.PARAM_UNSET)
     if not arg.required:
       chType = type(ch['value'])
-      if chType is not None:
+      if ch['value'] is not None:
         arg.type = type(ch['value'])
       arg.default = ch['value']
     if 'tip' in ch:
@@ -823,30 +824,6 @@ def popupFilePicker(parent=None, winTitle: str='', fileFilter: str='', existing=
   else:
     return None
 
-# Define at module level for mproc usage
-def _cvtImage(file, ext='png', replace=True):
-  if file.suffix[1:] == ext:
-    return
-  try:
-    cv.imwrite(str(file.with_suffix(f'.{ext}')), cv.imread(str(file)))
-    if replace:
-      file.unlink()
-  except Exception as ex:
-    return file, ex
-
-def convertImages(globstr:str='*.*', ext='png', replace=True, folder: FilePath=None):
-  if folder is None:
-    folder = Path()
-  folder = Path(folder)
-  files = list(folder.glob(globstr))
-  ret = mproc_apply(_cvtImage, files, 'Converting Files', extraArgs=(ext, replace))
-  errs = [f'{r[0].name}: {r[1]}' for r in ret if r is not None]
-  if errs:
-    print(
-      f'Conversion errors occurred in the following files:\n'
-      + '\n'.join(errs)
-    )
-
 def hierarchicalUpdate(curDict: dict, other: dict, replaceLists=False, uniqueListElements=False):
   """Dictionary update that allows nested keys to be updated without deleting the non-updated keys"""
   if other is None:
@@ -930,3 +907,26 @@ def naturalSorted(iterable):
 
     numbers = re.compile(r'(\d+)')
     return sorted(iterable, key=sortkey)
+
+
+class DummySignal:
+  """Useful for spoofing a qt connection that doesn't do anything"""
+  def connect(self, *args): pass
+  def disconnect(self, *args): pass
+  def emit(self, *args):
+    if self.capture:
+      self.emissions.append(args)
+
+  def __init__(self, capture=False):
+    self.emissions = []
+    self.capture = capture
+
+@contextmanager
+def makeDummySignal(obj, sigName: str, capture=False):
+  oldSig = getattr(obj, sigName)
+  try:
+    newSig = DummySignal(capture)
+    setattr(obj, sigName, newSig)
+    yield newSig
+  finally:
+    setattr(obj, sigName, oldSig)
