@@ -9,7 +9,7 @@ from pyqtgraph.Qt import QtCore, QtWidgets, QtGui
 from pyqtgraph.parametertree import Parameter
 from pyqtgraph.parametertree.parameterTypes import GroupParameter
 
-from utilitys.widgets import EasyWidget, DummySignal
+from utilitys.widgets import EasyWidget
 from . import PrjParam
 from .procwrapper import addStageToParam
 from .. import fns
@@ -573,7 +573,7 @@ class ParamEditor(QtWidgets.QDockWidget):
     # Kind of a hack: Make sure no sigValueChange is emmittedto until after setLimits
     oldSig = self.stateSelector.sigValueChanged
     if blockLoad:
-      self.stateSelector.sigValueChanged = DummySignal()
+      self.stateSelector.sigValueChanged = fns.DummySignal()
     try:
       self.stateSelector.setLimits(self.savedStates)
     finally:
