@@ -363,9 +363,11 @@ class FilePickerParameter(Parameter):
 class NoneParameter(parameterTypes.SimpleParameter):
 
   def __init__(self, **opts):
-    opts['type'] = 'str'
+    opts.update(readonly=True)
     super().__init__(**opts)
-    self.setWritable(False)
+
+  def _interpretValue(self, v):
+    return None
 
 parameterTypes.registerParameterType('nonetype',        NoneParameter,            override=True)
 parameterTypes.registerParameterType('NoneType',        NoneParameter,            override=True)
