@@ -1,4 +1,5 @@
 from __future__ import annotations
+from contextlib import ExitStack
 
 import copy
 import copy as _copy
@@ -16,9 +17,8 @@ from pyqtgraph.Qt import QtCore
 from pyqtgraph.parametertree import Parameter
 from pyqtgraph.parametertree.Parameter import PARAM_TYPES
 
-from . import misc
 from ._funcparse import funcToParamDict, RunOpts
-from .fns import nameFormatter
+from .fns import nameFormatter, makeDummySignal
 
 __all__ = ['ProcessIO', 'ProcessStage', 'NestedProcess', 'AtomicProcess', 'ParamContainer']
 
@@ -85,6 +85,15 @@ class ParamContainer(MutableMapping):
       deep.params[param] = Parameter.create(**val.saveState())
     return deep
 
+  def __str__(self):
+    pieces = []
+    if len(self.extras):
+      pieces.append(f'Extra: {self.extras}')
+    if len(self.params):
+      pieces.append(f'Params: {self.params}')
+    return '\n'.join(pieces)
+
+
 class ProcessIO(ParamContainer):
   """
   The object through which the processor pipeline communicates data. Inputs to one process
@@ -123,7 +132,7 @@ class ProcessIO(ParamContainer):
            f' inputs:\n{warnKeys}\n'
            f'Defaulting to `None` for those keys.', UserWarning)
     super().__init__(**kwargs)
-    self.keysFromPrevIO = set(self.keys()) - set(self.hyperParamKeys)
+    self.keysFromPrevIO = {k for (k, v) in self.items() if v is self.FROM_PREV_IO}
     self.hasKwargs = False
 
   @classmethod
@@ -146,25 +155,32 @@ class ProcessIO(ParamContainer):
     overriddenDefaults.setdefault('title', nameFormatter)
     funcDict = funcToParamDict(func, **overriddenDefaults)
 
-    outDict = {ch['name']: ch for ch in funcDict['children']
-               if not ch.get('ignore', False)}
+    outDict = {ch['name']: ch for ch in funcDict['children']}
     if ignoreKeys is None:
       ignoreKeys = []
     for kk in ignoreKeys:
-      outDict.pop(kk, None)
+      if kk not in overriddenDefaults:
+        outDict.pop(kk, None)
 
     values = [ch['value'] for ch in outDict.values()]
+    ignores = [ch.get('ignore', False) for ch in outDict.values()]
     if not interactive:
       # Recycle values only
       outDict = dict(zip(outDict, values))
 
     hasKwargs = outDict.pop('test for kwargs', False)
     hyperParamKeys = []
-    for (name, child), val in zip(outDict.items(), values):
+    for ii, (name, child) in enumerate(outDict.items()):
       # Unrepresentable parameters should be turned back into raw values
-      if val is RunOpts.PARAM_UNSET or (interactive and child['type'] not in PARAM_TYPES):
+      val = values[ii]
+      isHyperparam = (
+        val is not RunOpts.PARAM_UNSET
+        and name not in ignoreKeys
+        and not ignores[ii]
+      )
+      if not isHyperparam or (interactive and child['type'] not in PARAM_TYPES):
         outDict[name] = val
-      else:
+      elif isHyperparam:
         hyperParamKeys.append(name)
 
     cls._aliasOverlappingArgs(outDict)
@@ -218,18 +234,21 @@ class ProcessStage(ABC):
   def __eq__(self, other: NestedProcess):
     if not isinstance(other, NestedProcess):
       return False
-    return pg.eq(self.saveState(True, True), other.saveState(True, True))
+    try:
+      return pg.eq(self.saveState(True, True), other.saveState(True, True))
+    except TypeError:
+      # Default to false for types not comparable by pyqtgraph
+      return False
 
   def __hash__(self):
     return id(self)
 
-  def updateInput(self, prevIo: ProcessIO=None, graceful=False, allowExtra=False, **kwargs):
+  def updateInput(self, prevIo: ProcessIO=None, allowExtra=False, **kwargs):
     """
     Helper function to update current inputs from previous ones while ignoring leading
     underscores.
 
     :param prevIo: Io object to forward updated inputs here. Extra inputs can be supplied to **kwargs
-    :param graceful: If *True*, doesn't error on missing keys or bad update hierarchy
     :param allowExtra: Whether to allow setting of keys that didn't exist in the original input. This can be valid
       in the case where the underlying function accepts **kwargs
     """
@@ -372,7 +391,7 @@ class AtomicProcess(ProcessStage):
     if len(mainResultKeys) == 1:
       @wraps(func)
       def newFunc(*args, **kwargs):
-        return ProcessIO(**{mainResultKeys[0]: func(*args, **kwargs)})
+          return ProcessIO(**{mainResultKeys[0]: func(*args, **kwargs)})
     else:
       @wraps(func)
       def newFunc(*args, **kwargs):
@@ -408,7 +427,18 @@ class AtomicProcess(ProcessStage):
 
   def run(self, io: ProcessIO=None, disable=False, **runKwargs):
     disable = disable or self.disabled
-    self.updateInput(io, **runKwargs)
+    stack = ExitStack()
+    # Avoid parameter updates here from re-triggering a run by temporarily removing their changing signals
+    try:
+      for param in set(runKwargs).union(io or {}).intersection(self.input.params):
+        for sig in 'sigValueChanged', 'sigValueChanging':
+          stack.enter_context(makeDummySignal(self.input.params[param], sig))
+      self.updateInput(io, **runKwargs)
+    finally:
+      stack.close()
+    missingInputs = [f'"{kk}"' for (kk, vv) in self.input.items() if vv is self.input.FROM_PREV_IO]
+    if len(missingInputs):
+      raise KeyError(f'Missing one or more required inputs: {", ".join(missingInputs)}')
     try:
       # Use dicts instead of raw ProcessIO objects since pg params should
       # be converted to values before comparison
@@ -455,7 +485,7 @@ class AtomicProcess(ProcessStage):
       out[kk] = eval(expr, toMap)
     return ProcessIO(**out)
 
-  def updateInput(self, prevIo: ProcessIO=None, graceful=False, allowExtra=False, **kwargs):
+  def updateInput(self, prevIo: ProcessIO=None, allowExtra=False, **kwargs):
     allowExtra = allowExtra or self.hasKwargs
     if prevIo is None:
       prevIo = ProcessIO()
@@ -463,7 +493,6 @@ class AtomicProcess(ProcessStage):
     useIo.update(kwargs)
     useIo = self.computeArgMapping(self.inMap, useIo)
     selfFmtToUnfmt = {k.lstrip('_'): k for k in self.input}
-    requiredKeyFmt = {k for k, v in selfFmtToUnfmt.items() if v in self.input.keysFromPrevIO}
     prevIoKeyToFmt = {k.lstrip('_'): k for k in {**useIo}}
     setKeys = set()
     for fmtK, value in zip(prevIoKeyToFmt, useIo.values()):
@@ -476,11 +505,6 @@ class AtomicProcess(ProcessStage):
         self.input[fmtK] = value
         # Not possible for this to be required, so no need to add to set keys
 
-    missingKeys = (requiredKeyFmt - setKeys)
-    if missingKeys and not graceful:
-      # Some required keys were not given
-      raise KeyError(f'Missing following keys from {self}: {missingKeys}')
-
   def saveState(self, includeDefaults=False, includeMeta=False, **metaFilterOpts):
     def keepCond(k, v):
       # Can fail with unsavable types (e.g. numpy arrays)
@@ -541,24 +565,31 @@ class NestedProcess(ProcessStage):
     self.stages.append(process)
     return process
 
-  def updateInput(self, prevIo: ProcessIO=None, graceful=False, allowExtra=False, **kwargs):
+  def updateInput(self, prevIo: ProcessIO=None, allowExtra=False, **kwargs):
     """Drill down to the necessary stages when setting input values when setter values are dicts"""
     if prevIo is None:
       prevIo = ProcessIO()
     prevIo.update(kwargs)
+    # Make a copy to avoid in-place loss of keys
+    prevIo = prevIo.copy()
+    # List wrap instead of items() to avoid mutate error
     for kk in list(prevIo):
-      # List wrap instead of items() to avoid mutate error
       vv = prevIo[kk]
       if isinstance(vv, dict):
         matchStage = [s for s in self if s.name == kk]
-        if not matchStage and not graceful:
-          raise ValueError(f'Name {kk} doesn\'t match existing stages')
-        elif not matchStage:
+        if not matchStage:
+          # Since key doesn't match a stage name, assume it actually belongs to a parameter. This will be consumed below
           continue
-        matchStage[0].updateInput(**vv, graceful=graceful, allowExtra=allowExtra)
-        prevIo.pop(kk)
+        else:
+          # TODO: Apply to first matching stage or all stages with this name?
+          matchStage[0].updateInput(**vv, allowExtra=allowExtra)
+          # Eliminate this key from global updates since it was locally consumed
+          prevIo.pop(kk)
     # Normal input maps to first stage, so propagate all normal args there
-    self.stages[0].updateInput(prevIo, graceful=graceful, allowExtra=allowExtra)
+    # These are all args not matching a specific stage name. Dangerous to allow extra
+    # in an unspecified stage, so turn it false otherwise all stages will grow many keys
+    for stage in self.stages:
+      stage.updateInput(prevIo, allowExtra=False)
 
   def run(self, io: ProcessIO = None, disable=False, **runKwargs):
     _activeIo = ProcessIO() if not io else _copy.copy(io)
@@ -708,31 +739,4 @@ class NestedProcess(ProcessStage):
           if np.array_equal(info[key], lastInfo[key]):
             validInfo[key] = cls._DUPLICATE_INFO
       validInfos.append(validInfo)
-    return validInfos
-
-class ArgMapper(AtomicProcess):
-  """
-  Used to convert the named outputs of a previous stage into different names for process compatibility. Meant to be used
-  within a NestedProcess. Specify kwargs in the form of {outKeyName: inKeyName}.
-  """
-
-  def __init__(self, **kwargs):
-    # Just give a dummy function accepting overridable kwargs
-    kwargs['func'] = self.argMapper
-    super().__init__(**kwargs)
-    self.forwardOthers = True
-
-  def updateInput(self, prevIo: ProcessIO = None, graceful=False, allowExtra=True, **kwargs):
-    return super().updateInput(prevIo, graceful, allowExtra, **kwargs)
-
-  def argMapper(self, **kwargs):
-    if self.forwardOthers:
-      out = ProcessIO(**kwargs)
-      for kk in self.defaultInput.values():
-        # Remove keys that are mapped to new names
-        del out[kk]
-    else:
-      out = ProcessIO()
-    for kk, vv in self.defaultInput.items():
-      out[kk] = kwargs[vv]
-    return out
+    return validInfos
\ No newline at end of file
