@@ -1,6 +1,5 @@
 from __future__ import annotations
 
-import contextlib
 import html
 import logging
 import typing as t
@@ -10,13 +9,9 @@ from traceback import format_exception
 from warnings import warn
 import sys
 
-import cv2 as cv
 import numpy as np
 import pandas as pd
 import pyqtgraph as pg
-from matplotlib import cm
-from matplotlib.pyplot import colormaps
-from numpy.distutils.misc_util import is_sequence
 from pyqtgraph import GraphicsScene
 from pyqtgraph.Qt import QtWidgets, QtGui, QtCore
 from pyqtgraph.graphicsItems.LegendItem import ItemSample
@@ -404,7 +399,9 @@ class ButtonCollection(QtWidgets.QGroupBox):
       newBtn.setText('')
       newBtn.setIcon(QtGui.QIcon(str(btnOpts.opts['icon'])))
       tooltipText = btnOpts.addHelpText(btnOpts.name)
-    newBtn.setToolTip(forceRichText(tooltipText))
+    if tooltipText:
+      tooltipText = forceRichText(tooltipText)
+      newBtn.setToolTip(tooltipText)
     reg = params.pgregistered.ShortcutParameter.REGISTRY
     if reg is not None:
       reg.registerButton(btnOpts, newBtn, **kwargs)
@@ -466,7 +463,7 @@ class ButtonCollection(QtWidgets.QGroupBox):
 
 _layoutTypes = t.Union[t_e.Literal['H'], t_e.Literal['V']]
 class EasyWidget:
-  def __init__(self, children: t.Sequence,
+  def __init__(self, children: t.MutableSequence,
                layout: str=None, useSplitter=False, baseWidget: QtWidgets.QWidget=None):
     if baseWidget is None:
       baseWidget = QtWidgets.QWidget()
@@ -676,23 +673,33 @@ class ImageViewer(CompositionMixin, pg.PlotWidget):
     fontColor = 'black' if grayClr > 127 else 'white'
     self.pxColorLbl.setStyleSheet(f'background:{imColor.name()}; color:{fontColor}')
 
-  def setImage(self, imgSrc: t.Union[FilePath, np.ndarray]=None):
+  def setImage(self, imgSrc: t.Union[FilePath, np.ndarray]=None, stripAlpha=False):
     """
-    Allows the user to change the main image either from a filepath or array data
+    Allows the user to change the main image either from a filepath or array data.
+    Files can be any format accepted by QImage -> pg.imageToArray
+    Optionally strips the alpha channel from the image, if it exists (i.e. if its shape is MxNx4)
     """
     if isinstance(imgSrc, FilePath.__args__):
       # TODO: Handle alpha channel images. For now, discard that data
-      imgSrc = cv.imread(str(imgSrc), cv.IMREAD_UNCHANGED)
-      if imgSrc.ndim > 3:
-        # Alpha channels cause unexpected results for most image processors. Avoid this
-        # by chopping it off until otherwise needed
-        imgSrc = imgSrc[:,:,0:3]
-      if imgSrc.ndim == 3:
-        imgSrc = cv.cvtColor(imgSrc, cv.COLOR_BGR2RGB)
+      qtImg = QtGui.QImage(str(imgSrc))
+      nchans = qtImg.bitPlaneCount()//8
+      imgSrc = pg.imageToArray(qtImg, transpose=False)
 
+      if nchans >= 3:
+        chanOrder = [2,1,0]
+        # Alpha is always added by imageToArray, so chop it off if necessary
+        if nchans == 4:
+          chanOrder.append(3)
+        imgSrc = imgSrc[..., chanOrder]
+        # imageToArray creates bgr array, turn this into rgb. Also, some algorithms perform better
+        # with contiguous arrays, so make sure potential views after this operation remain contiguous
+        # Finally, force a copy to avoid crashing after qtImg (which owns the array) is garbage collected
+      imgSrc = imgSrc.copy(order='C')
     if imgSrc is None:
       self.imgItem.clear()
     else:
+      if stripAlpha and imgSrc.ndim > 2:
+        imgSrc = imgSrc[:,:,:3]
       self.imgItem.setImage(imgSrc)
 
   def widgetContainer(self, asMainWin=True, showTools=True, **kwargs):
@@ -758,10 +765,13 @@ class MaskCompositor(ImageViewer):
     # -----
     # Create properties
     # -----
+    self.masksParam = getParamChild(self.toolsEditor.params, 'Overlays')
+
     self.legend = CompositorLegend(offset=(5,5), horSpacing=5, brush='ccce', pen='k')
-
-    self.recordDf = pd.DataFrame(columns=['name', 'item', 'scatter', 'opacity']).set_index('name')
-
+    self.recordDf = pd.DataFrame(columns=['name', 'item', 'opacity', 'isMask']).set_index('name')
+    # Initialized in clearOverlays and updateLabelMap. Given here to avoid intellisense warnings
+    self.inUseLabelValues = self.scatterSer = self.backgroundValues = self.labelToNameMapping = None
+    self.updateLabelMap()
     self.legendFontArgs = {'size': '11pt', 'color': 'k', 'bold': True}
     self.curZ = 2
 
@@ -771,7 +781,6 @@ class MaskCompositor(ImageViewer):
     self.legend.setParentItem(self.plotItem)
     self.viewbox: pg.ViewBox = self.getViewBox()
 
-    self.masksParam = getParamChild(self.toolsEditor.params, 'Overlays')
     self.allVisible = True
     # for ax in 'left', 'bottom', 'right', 'top':
     #   self.mainImg.plotItem.hideAxis(ax)
@@ -794,6 +803,16 @@ class MaskCompositor(ImageViewer):
       self.toolsEditor.registerFunc(self.toggleAllVisible, namePath=('Overlays',))
     if img is not None:
       self.setImage(img)
+    self.clearOverlays()
+
+  @staticmethod
+  def _createLabelToNameMapping(data=None):
+    """Helper method to ensure proper dtypes when making a legend series. Helps avoid pitfalls with empty data, etc."""
+    if data is None:
+      data = []
+    ser = pd.Series(data, dtype=str)
+    ser.index = ser.index.astype('int64', copy=False)
+    return ser
 
   def _addItemCtrls(self, record: pd.Series):
     item = record['item']
@@ -817,29 +836,6 @@ class MaskCompositor(ImageViewer):
     super().setImage(*args, **kwargs)
     self._updateLegendPos()
 
-
-  def addMask(self, mask: np.ndarray, name:str=None, ignoreIfBlank=False, update=True):
-    if ignoreIfBlank and not np.any(mask):
-      return
-    newRecord = {}
-
-    name = self._getUniqueName(name)
-
-    curScat = pg.ScatterPlotItem(symbol='s', width=5)
-    self.legend.addItem(CompositorItemSample(curScat), name=name)
-    newRecord['scatter'] = curScat
-
-    toAdd = pg.ImageItem(mask)
-    toAdd.setZValue(self.curZ + 1)
-    self.viewbox.addItem(toAdd)
-    newRecord['item'] = toAdd
-
-    newRecord['opacity'] = -1
-
-    rec = pd.Series(newRecord, name=name)
-    self._addRecord(rec, update)
-    return rec
-
   def _addRecord(self, rec: pd.Series, update=True):
     self.recordDf = self.recordDf.append(rec)
     self.curZ += 1
@@ -850,25 +846,17 @@ class MaskCompositor(ImageViewer):
       self._updateGraphics()
 
   def addImageItem(self, item: pg.ImageItem, **kwargs):
-    kwargs.setdefault('scatter', None)
     kwargs.setdefault('opacity', -1)
     name = self._getUniqueName(kwargs.pop('name', None))
     update = kwargs.pop('update', True)
     newRecord = dict(item=item, **kwargs)
-
+    kwargs.setdefault('isMask', False)
     item.setZValue(self.curZ + 1)
     self.viewbox.addItem(item)
 
     rec = pd.Series(newRecord, name=name)
     self._addRecord(rec, update)
 
-  def addMasks(self, masks: t.List[np.ndarray], names: t.Sequence[t.Optional[str]]=None, ignoreIfBlank=False):
-    if names is None:
-      names = [None]*len(masks)
-    for mask, name in zip(masks, names):
-      self.addMask(mask, name, ignoreIfBlank, update=False)
-    self._updateGraphics()
-
   def _getUniqueName(self, baseName:str=None):
     if baseName is None:
       baseName = '[No Name]'
@@ -879,24 +867,60 @@ class MaskCompositor(ImageViewer):
       ii += 1
     return name
 
-  def addLabelMask(self, labelMask: np.ndarray, names='Label', ignoreBackground=True):
+  def addLabelMask(
+    self,
+    labelMask: np.ndarray,
+    name=None,
+    clearOverlays=False,
+    fullLabelMapInLegend=False,
+    **overlayKwargs
+  ):
     """
     Splits a label mask into its constituent labels and adds a mask for each unique label.
 
     :param labelMask: Grayscale label mask of integers
-    :param names: Base name of each label. E.g. if 3 labels are present in the image, the
-      resulting masks will be named <name> 1, <name> 2, <name 3>. Alternatively,
-      a list of names can be provided
-    :param ignoreBackground: Whether a mask should be added for 0-valued labels (typically
-      0 is used to denote background / non-label values)
+    :param name: Name to identify this mask in the property editor
+    :param ignoreValue: Label values matching ``ignoreValue`` will be transparent in the mask overlay.
+    :param ignoreIfBlank: If *True* and the whole mask is the background color, no action will be performed
+    :param clearOverlays: If *True*, `clearOverlays` will be called before adding this item. This is useful
+      if only one mask is viewed at a time
+    :param fullLabelMapInLegend: See ``updateLegendEntries`` -- this parameter is passed during legend creation
+    :param overlayKwargs: Keyword arguments passed to setOverlayProperties
     """
-    labels = np.unique(labelMask)
-    if ignoreBackground:
-      labels = labels[labels > 0]
-    newMasks = [labelMask == label for label in labels]
-    if not is_sequence(names):
-      names = [f'{names} {ii}' for ii in labels]
-    self.addMasks(newMasks, names)
+    if clearOverlays and len(self.recordDf):
+      # Reuse item if possible
+      item = self._reuseLastOverlay(labelMask, name)
+    else:
+      item = None
+    maskValues = np.unique(labelMask.ravel())
+    # Booleans don't mix well with int series indexing/logic
+    # Floats are completely wonky
+    if np.issubdtype(maskValues.dtype, np.bool_):
+      maskValues = maskValues.astype('uint8')
+    elif not np.issubdtype(maskValues.dtype, np.integer):
+      raise ValueError(f'Overlays can only be created using integer dtypes. Encountered "{maskValues.dtype}"')
+
+    self.inUseLabelValues = np.union1d(self.inUseLabelValues, maskValues)
+    self.updateLegendEntries(useFullLabelMap=fullLabelMapInLegend)
+    if item is None:
+      self.addImageItem(pg.ImageItem(labelMask), name=name, isMask=True, opacity=-1, update=False)
+    self.propertiesProc.run(**overlayKwargs)
+  
+  def _reuseLastOverlay(self, newImageData=None, newName=None, isMask=True):
+    """
+    When a set of images must be overlaid separately, it is more efficient to reuse an
+    image item compared to constantly adding and removing them from a scene.
+    Assumes at least one record is already present in ``self.recordDf``
+    """
+    self.clearOverlays(self.recordDf.index[:-1])
+    # Use two steps to update name so conflicts are avoided
+    self.recordDf.index = [None]
+    self.recordDf.index = [self._getUniqueName(newName)]
+    rec = self.recordDf.iloc[-1]
+    rec['isMask'] = isMask
+    item = rec['item']
+    item.setImage(newImageData)
+    return item
 
   def setLegendFontStyle(self, startItemIdx=0, **lblTxtArgs):
     for item in self.legend.items[startItemIdx:]:
@@ -904,8 +928,54 @@ class MaskCompositor(ImageViewer):
         if isinstance(single_item, pg.LabelItem):
           single_item.setText(single_item.text, **lblTxtArgs)
 
-  @dynamicDocstring(cmapVals=colormaps())
-  def setOverlayProperties(self, colormap='Set1', opacity=0.5):
+  def updateLegendEntries(self, useFullLabelMap=False, updateOverlays=False):
+    """
+    :param useFullLabelMap: If *True*, guarantees that every entry in the label->name mapping will be present
+      in the legend.
+    :param updateOverlays: If *True*, the image overlay properties will be refereshed
+    """
+    if useFullLabelMap:
+      labels = np.union1d(self.inUseLabelValues, self.labelToNameMapping.index)
+    else:
+      labels = self.inUseLabelValues
+    labels = np.setdiff1d(labels, self.backgroundValues)
+    names = self.labelToNameMapping.reindex(labels)
+    needsEntry = names.isna()
+    names[needsEntry] = names.index[needsEntry].map(str)
+    self.legend.clear()
+    self.scatterSer = pd.Series([None]*len(names), index=names.index, dtype=object)
+    # Make sure alpha is not carried over
+    for value, name in names.iteritems():
+      scat = pg.ScatterPlotItem(symbol='s', width=5)
+      self.legend.addItem(CompositorItemSample(scat), name=name)
+      self.scatterSer.at[value] = scat
+    if updateOverlays:
+      self.propertiesProc.run()
+
+  def updateLabelMap(
+    self, 
+    labelMap: dict | pd.Series=None,
+    backgroundValues=0,
+    updateOverlays=False
+  ):
+    """
+    :param labelMap: Series or dict-like with integer key and string value. Gives a legend name to each label in
+      the image. Unspecified labels will be named by their numeric value, e.g. 34 will be '34'. Note that this
+      will update the global legend, meaning if {10 -> 'test'} is a legend entry, it will overwrite the old
+      legend value for 10.
+    :param backgroundValues: Represents label mask values that should be transparent in the overlay
+    """
+    if labelMap is None:
+      labelMap = {}
+    if np.isscalar(backgroundValues):
+      backgroundValues = [backgroundValues]
+    self.backgroundValues = np.array(backgroundValues)
+    self.labelToNameMapping = self._createLabelToNameMapping(labelMap)
+    if updateOverlays:
+      self._updateGraphics()
+
+  @dynamicDocstring(cmapVals=pg.colormap.listMaps())
+  def setOverlayProperties(self, colormap='magma', opacity=0.6, fullLabelMapColors=False):
     """
     Sets overlay properties
     :param colormap:
@@ -914,29 +984,48 @@ class MaskCompositor(ImageViewer):
     :param opacity:
       limits: [0,1]
       step: 0.1
+    :param fullLabelMapColors: If *True*, enough colors will be generated for all labels in
+      ``self.labelToNameMapping``. If *False*, only enough colors will be generated for the labels in the
+      added masks.
+      ignore: True
     """
-    maskNames = np.array([s is not None for s in self.recordDf['scatter']], dtype=bool)
-    names = self.recordDf.index[maskNames]
-    numMasks = len(names)
-    cmap = cm.get_cmap(colormap, numMasks)
-    colors = cmap(np.arange(numMasks))
-    colors = (colors*255).astype('uint8')
-    backgroundClr = [0,0,0,0]
-    # Filter out names without associated scatters
-    for name, color in zip(names, colors):
+    maskIdxs = self.recordDf['isMask'].to_numpy(bool)
+    try:
+      cmap = pg.colormap.get(colormap)
+    except FileNotFoundError:
+      cmap = pg.colormap.get(colormap, source='matplotlib')
+    labelValues = self.scatterSer.index.values
+    if fullLabelMapColors:
+      labelValues = np.union1d(labelValues, self.labelToNameMapping.index.values)
+    if not len(labelValues):
+      return
+    if cmap is None:
+      raise ValueError(f'Invalid colormap: {colormap}')
+    colors = cmap.getLookupTable(nPts=len(labelValues), alpha=True)
+    colors[:, -1] = opacity*255
+    # If a labelmap has entries, try to scale the colormap to contain all possibilities
+    numEntries = max(
+      np.max(labelValues, initial=-1),
+      np.max(self.labelToNameMapping.index.values, initial=-1)
+    ) + 1
+    lut = np.zeros((numEntries, 4), dtype='uint8')
+    lut[labelValues] = colors
+    lut[self.backgroundValues] = 0
+    for idx, scat in self.scatterSer.iteritems():
       # Make sure alpha is not carried over
+      color = lut[idx, :-1]
       brush, pen = pg.mkBrush(color), pg.mkPen(color)
-      self.recordDf.at[name, 'scatter'].setData(symbol='s', brush=brush, pen=pen, width=5)
-    for name, color in zip(names, colors):
-      curLut = np.array([backgroundClr, color])
-      itemOpacity = self.recordDf.at[name, 'opacity']
+      scat.setData(symbol='s', brush=brush, pen=pen, width=5)
+    for _, maskRec in self.recordDf[maskIdxs].iterrows():
+      itemOpacity = maskRec['opacity']
       if itemOpacity < 0:
         itemOpacity = opacity
-      self.recordDf.at[name, 'item'].setOpts(levels=[0,1], lut=curLut, opacity=itemOpacity)
+      maskRec['item'].setOpts(lut=lut, opacity=itemOpacity, levels=[0, numEntries-1])
     self.setLegendFontStyle(**self.legendFontArgs)
+    self.legend.update()
 
     # Handle all non-mask items now
-    for name, remainingRec in self.recordDf[~maskNames].iterrows():
+    for name, remainingRec in self.recordDf[~maskIdxs].iterrows():
       curOpacity = remainingRec['opacity']
       if curOpacity < 0:
         curOpacity = opacity
@@ -949,12 +1038,24 @@ class MaskCompositor(ImageViewer):
   def _updateGraphics(self):
     self.propertiesProc.run()
 
-  def clearOverlays(self):
-    for imgItem in self.recordDf['item']:
+  def clearOverlays(self, clearIdxs=None):
+    """
+    Clears image item overlays by index. If No index is specified, all overlays are removed
+    :param clearIdxs: Indexes of overlays to remove
+    ignore: True
+    """
+    if clearIdxs is None:
+      clearIdxs = self.recordDf.index
+    for imgItem in self.recordDf.loc[clearIdxs, 'item']:
       self.viewbox.removeItem(imgItem)
-    self.recordDf = self.recordDf.drop(self.recordDf.index)
-    self.legend.clear()
-    self.toolsEditor.params.child('Overlays').clearChildren()
+    keepIdxs = np.setdiff1d(self.recordDf.index, clearIdxs)
+    self.recordDf: pd.DataFrame = self.recordDf.loc[keepIdxs].copy()
+    self.inUseLabelValues = np.array([], 'int64')
+    self.updateLegendEntries()
+    overlayParam = self.toolsEditor.params.child('Overlays')
+    
+    for idx in clearIdxs:
+      overlayParam.child(idx).remove()
 
   def toggleAllVisible(self):
     for param in self.masksParam:
@@ -962,7 +1063,7 @@ class MaskCompositor(ImageViewer):
         param.setValue(not self.allVisible)
     self.allVisible = not self.allVisible
 
-  def save(self, saveFile: FilePath='', cropToViewbox=False, toClipboard=False, floatLegend=False) -> bool:
+  def save(self, saveFile: FilePath='', cropToViewbox=False, toClipboard=False, floatLegend=False):
     """
     :param saveFile:
       helpText: Save destination. If blank, no file is created.
@@ -1033,6 +1134,22 @@ class MaskCompositor(ImageViewer):
       self.legend.setParentItem(self.plotItem)
       self.legend.autoAnchor(oldPos, relative=False)
 
+  def __getstate__(self):
+    ret = dict(
+      image=self.imgItem.image,
+      legendVisible=self.legend.isVisible(),
+    )
+    if len(self.recordDf):
+      ret['recordDf'] = self.recordDf
+
+  def __setstate__(self, state):
+    self.__init__(state['image'])
+    self.legend.setVisible(state['legendVisible'])
+    if 'recordDf' in state:
+      for _, rec in state['recordDf'].iteritems():
+        self._addRecord(rec, False)
+      self._updateGraphics()
+
 class PandasTableModel(QtCore.QAbstractTableModel):
   """
   Class to populate a table view with a pandas dataframe
@@ -1198,30 +1315,6 @@ class PandasTableModel(QtCore.QAbstractTableModel):
         except (TypeError, ValueError):
           continue
 
-
-class DummySignal:
-  """Useful for spoofing a qt connection that doesn't do anything"""
-  def connect(self, *args): pass
-  def disconnect(self, *args): pass
-  def emit(self, *args):
-    if self.capture:
-      self.emissions.append(args)
-
-  def __init__(self, capture=False):
-    self.emissions = []
-    self.capture = capture
-
-@contextlib.contextmanager
-def makeDummySignal(obj, sigName: str, capture=False):
-  oldSig = getattr(obj, sigName)
-  try:
-    newSig = DummySignal(capture)
-    setattr(obj, sigName, newSig)
-    yield newSig
-  finally:
-    setattr(obj, sigName, oldSig)
-
-
 class QtAppHandler(logging.Handler):
   _weakWin: weakref.ReferenceType = None
 
